/*
 * (c) 2019, SWD Embedded Systems Limited, http://www.kpda.ru
 */


#include "externs.h"


/* Description: return current base clock(not dh/dl)
 * In:
 *      dev - device handler
 * Out:
 *      current clock in Hz */
uint64_t get_baytrail_clock( DEV_8250* dev )
{
    uint32_t clock = mem_read32( dev->ext_regs + BTUART_PRIV_CLOCK_PARAMS );
    uint64_t m     = BTUART_PRIV_CLOCK_PARAMS_GET_M( clock );
    uint64_t n     = BTUART_PRIV_CLOCK_PARAMS_GET_N( clock );

    if ( clock == 0 )
        return (0);

    /* Fbase = 100000000 (Hz)
     * Formula to calculate base clock : ( FBase * M ) / N */
    return (FBASE * m) / n;
}


/* Description: set base clock(not dh/dl)
 * In:
 *      dev - device handler
 *      m - multiplier ( see formula )
 *      n - divider ( see formula )
 * Out:
 *      0 if setting success, other for error */
int set_baytrail_clock( DEV_8250* dev, int m , int n )
{
    uint32_t clock = mem_read32( dev->ext_regs + BTUART_PRIV_CLOCK_PARAMS );

    if ( clock == 0 )
        return (-1);

    clock = BTUART_PRIV_CLOCK_PARAMS_SET_M( m, clock );
    clock = BTUART_PRIV_CLOCK_PARAMS_SET_N( n, clock );
    mem_write32( dev->ext_regs + BTUART_PRIV_CLOCK_PARAMS, clock );

    return (0);
}
