/*
 * (c) 2019, SWD Embedded Systems Limited, http://www.kpda.ru
 */


#ifndef __BAYTRIL_H__
#define __BAYTRIL_H__

#define BTUART_PCI_VENDOR       0x8086
#define BTUART_PCI_DEVICE_0     0xF0A
#define BTUART_PCI_DEVICE_1     0xF0C

/* Count of extend registers. Full memory size = 0x824*/
#define BTUART_REG_CNT          2084

/* FBase clock default - 100MHz */
#define FBASE                   100000000UL

/* Regno range - {0 - 15} */
#define BTUART_SHADOW_REG( regnum )                         ( regnum + 0x30 )
/* FIFO access register */
#define BTUART_FAR                                          0x1C
/* Transmit FIFO register */
#define BTUART_TFR                                          0x1D
/* Receive FIFO write register */
#define BTUART_RFW                                          0x1E
/* UART status register */
#define BTUART_USR                                          0x1F
/* Transmit FIFO level */
#define BTUART_TFL                                          0x20
/* Receive FIFO level */
#define BTUART_RFL                                          0x21
/* Software reset register */
#define BTUART_SRR                                          0x22
/* Shadow request to send */
#define BTUART_SRTS                                         0x23
/* Shadow break control */
#define BTUART_SBCR                                         0x24
/* Shadow DMA mode */
#define BTUART_SDMAM                                        0x25
/* Shadow FIFO enable */
#define BTUART_SFE                                          0x26
/* Shadow request to send */
#define BTUART_SRTS2                                        0x27
/* Shadow TX empty trigger */
#define BTUART_STET                                         0x28
/* Halt tx */
#define BTUART_HTX                                          0x29
/* DMA software acknowledge */
#define BTUART_DMASA                                        0x2A
/* Component parameter register */
#define BTUART_CPR                                          0x3D
    #define BTUART_CPR_FIFO_mode_mask                       (0xFF << 16)
    #define BTUART_CPR_GET_FIFO_mode( val )                 (val & BTUART_CPR_FIFO_mode_mask >> 16)
        #define BTUART_FIFO_SIZE( val )                     (val * 32)
    #define BTUART_CPR_dma_extra                            (1 << 13)
    #define BTUART_CPR_add_encoded_params                   (1 << 12)
    #define BTUART_CPR_shadow                               (1 << 11)
    #define BTUART_CPR_FIFO_stat                            (1 << 10)
    #define BTUART_CPR_FIFO_access                          (1 << 9)
    #define BTUART_CPR_additional_feat                      (1 << 8)
    #define BTUART_CPR_sir_lp_mode                          (1 << 7)
    #define BTUART_CPR_sir_mode                             (1 << 6)
    #define BTUART_CPR_thre_mode                            (1 << 5)
    #define BTUART_CPR_afce_mode                            (1 << 4)
    #define BTUART_CPR_apb_data_width                       (0x3)
/* Uart component version */
#define BTUART_UCV                                          0x3E
/* Component type register */
#define BTUART_CTR                                          0X3F
/* Private clock params */
#define BTUART_PRIV_CLOCK_PARAMS                            0x200
    #define BTUART_PRIV_CLOCK_PARAMS_clk_en                 0x1
    #define BTUART_PRIV_CLOCK_PARAMS_clk_update             (0x1 << 31)
    #define BTUART_PRIV_CLOCK_PARAMS_GET_M( val )           ((val >> 1) & 0x7FFF)
    #define BTUART_PRIV_CLOCK_PARAMS_GET_N( val )           ((val >> 16) & 0x7FFF)
    #define BTUART_PRIV_CLOCK_PARAMS_SET_M( val,regvar )    ( (regvar & ~0xFFFE) | ((val & 0x7FFF) << 1 ))
    #define BTUART_PRIV_CLOCK_PARAMS_SET_N( val,regvar )    ( (regvar & ~0x7FFF0000) | ((val & 0x7FFF) << 16 ))
/* Software resets */
#define BTUART_RESETS                                       0x201
/* General purpose register */
#define BTUART_GENERAL                                      0x202
/* UART byte count */
#define BTUART_BYTE_COUNT                                   0x206
/* overflow intr stat */
#define BTUART_OVERFLOW_INTR                                0x208
    #define BTUART_OVERFLOW_INTR_STAT                       (0x1 << 0)
    #define BTUART_OVERFLOW_INTR_ENABLE                     (0x1 << 1)


uint64_t get_baytrail_clock( DEV_8250* dev );
int set_baytrail_clock( DEV_8250* dev, int m , int n );

#endif
