/*
 * (c) 2010-2018, SWD Embedded Systems Limited, http://www.kpda.ru
 */


#ifdef DEFN
    #define EXT
    #define INIT1(a)    = { a }
#else
    #define EXT extern
    #define INIT1(a)
#endif


#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <malloc.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/neutrino.h>
#include <termios.h>
#include <devctl.h>
#include <sys/dcmd_chr.h>
#include <sys/iomsg.h>
#include <atomic.h>
#include <hw/inout.h>
#include <hw/8250.h>
#include <sys/mman.h>
#include <sys/io-char.h>
#include <sys/slog.h>
#include <sys/slogcodes.h>
#include <variant.h>


#if defined( VARIANT_pa6t ) || defined( VARIANT_tegra2 )
    /* enable workarounds for bugs in the UARTS of early PA6T-1682's */
    /* Note: The implementation fo the 8250 on the nVIDIA Tegra2
             behave the same way as the workaround for the PA6T.
             We will reuse the same code unless there is some
             difference between both implementation. */
    #define PA6T_WORKAROUND
#endif

#if defined( VARIANT_mpc8540 )
    #define MPC850_WORKAROUND
#endif

#if defined(VARIANT_jace5)
    #define TL16C752B_WORKAROUND
#endif

#ifndef DEV_8250            /* Can be defined in variant.h to override default */
typedef struct dev_8250 {
    TTYDEV          tty;
    struct dev_8250 *next;
    unsigned        intr;
    unsigned        clk;
    unsigned        div;
    unsigned char   rx_fifo;                        /* rx fifo size */
    unsigned char   tx_fifo;                        /* tx fifo size */
    unsigned char   fcr;                            /* FCR is write-only register, so keep local copy for read-modify-write */
    unsigned        dev_id;
    
#if defined( PA6T_WORKAROUND )
    unsigned char   irr_fiddle;                     /* for PA6T-1682 workaround */
    unsigned char   tx_empty_disable;               /* for PA6T-1682 workaround */
#endif

    uintptr_t       port[REG_TOTAL];

    uint32_t        *ext_regs;                      /* Extended UART registers */
    uint32_t        (*read_8250)( uintptr_t port ); /* HW access function pointers */
    void            (*write_8250)( uintptr_t port, uint32_t val );
} DEV_8250;
#endif

struct dev_list {
    struct dev_list *next;
    DEV_8250        *device;
    int             iid;
};

EXT TTYCTRL         ttyctrl;
EXT struct dev_list *devices;

struct pci_list_t {
    int             vendorID;
    int             deviceID;
};

#define MAX_DEVICES 16
#define FIFO_SIZE   16


#include "proto.h"
#include "baytrail.h"


#ifndef IE_SET_ALL
    #define IE_SET_ALL  (IE_RX | IE_TX | IE_LS | IE_MS)
#endif

#ifndef IE_CLR_ALL
    #define IE_CLR_ALL  0x0
#endif
