/*
 * (c) 2022, SWD Embedded Systems Limited, http://www.kpda.ru
 */


#include <stdio.h>
#include <sys/mman.h>
#include <gulliver.h>
#include "externs.h"


int exar_dev_init( struct pci_dev_info *dev, exar_dev_t *exdev )
{
    /* Disable PCIe Active-state power managment */
    if ( exdev->disable_aspm )
    {
        uint8_t byte = 0;
        pci_write_config8( dev->BusNumber, dev->DevFunc, 0x90, 1, &byte );
    }

    if ( (dev->Class != UART_DEV_CLASS) && (dev->Class != EXAR_DEV_CLASS) )
    {
        devc_err( "Warning: incorrect device type. Class code: 0x%X instead of expected 0x%X or 0x%X\n",
                  dev->Class, UART_DEV_CLASS, EXAR_DEV_CLASS );
        /* return (-1); */
    }

    exdev->pci_dev = dev;

    if ( (exdev->uart_mem = mmap_device_memory( NULL, exdev->pci_dev->BaseAddressSize[0],
                                                PROT_READ | PROT_WRITE | PROT_NOCACHE, 0,
                                                PCI_MEM_ADDR( exdev->pci_dev->CpuBaseAddress[0] ))) == MAP_FAILED )
    {
        devc_err( "Cannot map global uart region" );
        return (-1);
    }

    if ( exdev->uart_cnt < 0 )
    {
        if ( (dev->VendorId == EXAR_VID) && (dev->DeviceId == EXAR_XR17D154_DID) )
            exdev->uart_cnt = 4;
        else if ( (dev->VendorId == EXAR_VID) && (dev->DeviceId == EXAR_XR17D158_DID) )
            exdev->uart_cnt = 8;
        else if ( (dev->VendorId == ADVANTECH_VID) && (dev->DeviceId == ADVANTECH_PCI1612_DID_B || dev->DeviceId == ADVANTECH_PCI1612_DID_1B) )
            exdev->uart_cnt = 4;
        else
            exdev->uart_cnt = UART_DEFAULT_CH_COUNT;
    }

    uint32_t imi = exar_read32( exdev, EXAR_CONFIG_REG_BASE + UART_CONFIG_REG_ID_MPIO_INT_OFFSET );

    devc_dbg( "Exar device attached. DVID: 0x%X , Revision: 0x%X", UART_GET_DVID( imi ), UART_GET_REVISION( imi ) );

    return (0);
}


uint32_t exar_read32( exar_dev_t *dev, uintptr_t offset )
{
    return *((uint32_t *)(dev->uart_mem + offset));
}


uint8_t exar_read8( exar_dev_t *dev, uintptr_t offset )
{
    return *((uint8_t *)(dev->uart_mem + offset));
}


void exar_write32( exar_dev_t *dev, uintptr_t offset, uint32_t val )
{
    *(uint32_t *)(dev->uart_mem + offset) = val;
}


void exar_write8( exar_dev_t *dev, uintptr_t offset, uint8_t val )
{
    *(uint8_t *)(dev->uart_mem + offset) = val;
}


uint64_t exar_read64( exar_dev_t *dev, uintptr_t offset )
{
    return *((uint64_t *)(dev->uart_mem + offset));
}


void exar_write64( exar_dev_t *dev, uintptr_t offset, uint64_t val )
{
    *(uint64_t *)(dev->uart_mem + offset) = val;
}
